﻿using AutoMapper;
using Microsoft.EntityFrameworkCore;
using Org.BouncyCastle.Math.EC.Rfc7748;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.Services
{
    public class ForumService : IForumService
    {
        private readonly Tsi1Context _context;

        private readonly IMapper _mapper;

        public ForumService(Tsi1Context context, IMapper mapper)
        {
            _context = context;
            _mapper = mapper;
        }

        public async Task<ServiceResult<int>> Create(ForumCreateDto newForum)
        {
            var result = new ServiceResult<int>();
            var forum = _mapper.Map<Forum>(newForum);

            _context.Forums.Add(forum);
            await _context.SaveChangesAsync();
   
            result.Data = forum.Id;
            return result;
        }

        public async Task<ServiceResult<Forum>> Delete(int forumId)
        {
            var result = new ServiceResult<Forum>();

            var forum = await _context.Forums.FirstOrDefaultAsync(x => x.Id == forumId);

            if (forum == null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.ForumDoesNotExist, forumId);
                return result;
            }

            _context.Forums.Remove(forum);

            await _context.SaveChangesAsync();

            return result;
        }

        public async Task<ServiceResult<List<ForumPreviewDto>>> GetForums(int sectionId)
        {
            var result = new ServiceResult<List<ForumPreviewDto>>();

            var section = await _context.Sections
                .Where(x => x.Id == sectionId)
                .Include(x => x.SectionItems)
                    .ThenInclude(x => x.SectionItemType)
                .Include(x => x.SectionItems)
                    .ThenInclude(x => x.Forum)
                .FirstOrDefaultAsync();

            if (section == null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.SectionDoesNotExist, sectionId);
                return result;
            }

            var forums = section.SectionItems
                .Where(x => x.SectionItemType.Name == SectionItemTypes.Forum)
                .Select(x => x.Forum)
                .ToList();

            var forumDtos = _mapper.Map<List<ForumPreviewDto>>(forums);

            result.Data = forumDtos;

            return result;
        }

        public async Task<ServiceResult<List<string>>> GetSubscribedUsers(int forumId)
        {
            var result = new ServiceResult<List<string>>();

            var userEmails = await _context.ForumUsers
                .Where(x => x.ForumId == forumId)
                .Select(x => x.User.Email)
                .ToListAsync();

            result.Data = userEmails;

            return result;
        }

        public async Task<ServiceResult<List<bool>>> Subscribe(int forumId, int userId)
        {
            var result = new ServiceResult<List<bool>>();

            var forum = await _context.Forums.FirstOrDefaultAsync(x => x.Id == forumId);

            if (forum == null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.ForumDoesNotExist, forumId);
                return result;
            }

            var user = await _context.Users.FirstOrDefaultAsync(x => x.Id == userId);

            if (user == null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.UserDoesNotExist, userId);
                return result;
            }

            var existingForumUser = await _context.ForumUsers
                .FirstOrDefaultAsync(x => x.ForumId == forum.Id && x.UserId == user.Id);

            if (existingForumUser != null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.DuplicateForumUser, user.Username, forum.Name);
                return result;
            }

            var forumUser = new ForumUser
            {
                ForumId = forumId,
                UserId = userId,
            };

            _context.ForumUsers.Add(forumUser);

            await _context.SaveChangesAsync();

            return result;
        }
    }
}
