﻿using AutoMapper;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.Services
{
    public class SectionService : ISectionService
    {
        private readonly Tsi1Context _context;
        private readonly IMapper _mapper;
        private readonly ISectionItemService _sectionItemService;

        public SectionService(Tsi1Context context, IMapper mapper, ISectionItemService sectionItemService)
        {
            _context = context;
            _mapper = mapper;
            _sectionItemService = sectionItemService;
        }

        public async Task<ServiceResult<int>> Create(SectionCreateDto newSection)
        {
            var result = await this.CreateValidations(new List<SectionCreateDto>() { newSection });
            if (result.HasError)
            {
                return result;
            }

            var section = _mapper.Map<Section>(newSection);
            _context.Sections.Add(section);
            await _context.SaveChangesAsync();

            result.Data = section.Id;
            return result;
        }

        public async Task<ServiceResult<int>> Create(List<SectionCreateDto> newSections)
        {
            var result = await this.CreateValidations(newSections);
            if (result.HasError)
            {
                return result;
            }

            var sections = _mapper.Map<List<Section>>(newSections);
            _context.Sections.AddRange(sections);
            await _context.SaveChangesAsync();

            return result;
        }

        public async Task<ServiceResult<bool>> Delete(int sectionId)
        {
            var result = new ServiceResult<bool>();

            var section = await _context.Sections.FirstOrDefaultAsync(x => x.Id == sectionId);
            if (section == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.SectionDoesNotExist, sectionId));
                return result;
            }

            _context.Sections.Remove(section);
            await _context.SaveChangesAsync();
            return result;
        }

        public async Task<ServiceResult<bool>> Modify(int sectionId, string name)
        {
            var result = new ServiceResult<bool>();

            var section = await _context.Sections.FirstOrDefaultAsync(x => x.Id == sectionId);
            if (section == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.SectionDoesNotExist, sectionId));
                return result;
            }

            section.Name = name;
            await _context.SaveChangesAsync();
            return result;
        }

        public async Task<ServiceResult<bool>> OrderSections(List<OrderDto> orderDtos)
        {
            var result = new ServiceResult<bool>();

            var sectionIds = orderDtos.Select(x => x.Id);
            var orders = orderDtos.Select(x => x.Order).Distinct();

            if (orders.Count() != orderDtos.Count())
            {
                result.HasError = true;
                result.AddMessage(ErrorMessages.DuplicateSectionOrder);
                return result;
            }

            var sections = await _context.Sections
                .Where(x => sectionIds.Contains(x.Id))
                .ToListAsync();

            foreach (var orderDto in orderDtos)
            {
                var section = sections.FirstOrDefault(x => x.Id == orderDto.Id);
                if (section == null)
                {
                    result.HasError = true;
                    result.AddMessage(string.Format(ErrorMessages.SectionDoesNotExist, orderDto.Id));
                }

                section.Order = orderDto.Order;
            }

            if (!result.HasError)
            {
                await _context.SaveChangesAsync();
            }

            return result;
        }

        public async Task<ServiceResult<int>> CreateValidations(List<SectionCreateDto> sections, bool isFromCreateCourse = false)
        {
            var result = new ServiceResult<int>();

            if (!isFromCreateCourse)
            {
                var courseId = sections.Select(x => x.CourseId).SingleOrDefault();
                var course = await _context.Courses.AsNoTracking()
                    .SingleOrDefaultAsync(x => x.Id == courseId);

                if (course == null)
                {
                    result.HasError = true;
                    result.AddMessage(string.Format(ErrorMessages.CourseDoesNotExist, courseId));
                    return result;
                } 
            }

            var sectionItems = sections.SelectMany(x => x.SectionItems).ToList();
            result = await _sectionItemService.CreateValidations(sectionItems, true);

            return result;
        }
    }
}
